from odoo.upgrade import util

def migrate(cr, version):
    # Force reload quotation templates
    cr.execute("""
        SELECT id,
               (
                SELECT module||'.'||name
                  FROM ir_model_data
                 WHERE module = 'openerp_enterprise'
                   AND model = 'sale.order.template' AND res_id = sale_order_template.id
                 LIMIT 1
               ) AS xmlid
          FROM sale_order_template
         WHERE active = true
    """)
    template_ids = []
    for quotmpl_id, quotmpl_xmlid in cr.fetchall():
        if not quotmpl_xmlid:
            continue
        template_ids.append(quotmpl_id)
        util.update_record_from_xml(cr, quotmpl_xmlid)
    cr.flush()

    # Get the plan created by the standard upgrade for those templates
    # - they have the same name as the qutotation template
    # - for upgrade plan has the same frequence as the templates default plan,
    #   update the sale.order to update the default plan then remove duplicates
    # - for other upgrade plan, archive them - we only want to have 1 active
    #   plan per quotation template
    cr.execute("""
        SELECT sot.id,
               sot.plan_id AS default_plan_id,
               ssp.id AS upgrade_plan_id,
               (
                  CASE
                    WHEN ssp.billing_period_unit = sot_plan.billing_period_unit
                         AND ssp.billing_period_value = sot_plan.billing_period_value
                         THEN true
                    ELSE false
                  END
               ) AS is_duplicate
          FROM sale_order_template sot
          JOIN sale_subscription_plan ssp ON (ssp.name->>'en_US' = sot.name)
          JOIN sale_subscription_plan sot_plan ON (sot.plan_id = sot_plan.id)
         WHERE sot.id IN %s
           AND ssp.id != sot.plan_id
    """, (tuple(template_ids),))
    dups = {}
    to_archive = []
    for quotmpl_id, default_plan_id, upgrade_plan_id, is_duplicate in cr.fetchall():
        if is_duplicate:
            dups[upgrade_plan_id] = default_plan_id
        else:
            to_archive.append(upgrade_plan_id)

    if dups:
        when_then = "\n".join(
            "WHEN plan_id = %d THEN %d" % (duplicate_id, target_id)
            for duplicate_id, target_id in dups.items()
        )
        query = cr.mogrify(f"""
            UPDATE sale_order
               SET plan_id = CASE
                  {when_then}
                  ELSE plan_id
               END
            WHERE plan_id IN %s
        """, (tuple(dups),)).decode()
        util.explode_execute(cr, query, "sale_order")

        # Move manually created pricing from duplicate to target plan
        # (excluding duplicate pricing - have same product_template_id/pricelist_id/plan_id)
        for duplicate_id, target_id in dups.items():
            cr.execute("""
                UPDATE sale_subscription_pricing pr
                    SET plan_id = %s
                    WHERE NOT EXISTS (
                        SELECT 1
                            FROM sale_subscription_pricing ssp2
                        WHERE ssp2.product_template_id = pr.product_template_id
                            AND ssp2.pricelist_id = pr.pricelist_id
                            AND ssp2.plan_id = %s
                    )
                    AND plan_id = %s
            """, (target_id, target_id, duplicate_id))

        cr.execute("DELETE FROM sale_subscription_pricing WHERE plan_id IN %s", (tuple(dups),))
        cr.execute("DELETE FROM sale_subscription_plan WHERE id IN %s", (tuple(dups),))

    if to_archive:
        cr.execute("UPDATE sale_subscription_plan SET active = false WHERE id IN %s" % (tuple(to_archive),))
    cr.execute("""
        UPDATE sale_subscription_plan
           SET active = false
         WHERE name->>'en_US' ~ 'Success Pack (Standard|Custom|Basic|Pro)'
           AND active = true
    """)

    # final cleanup: we now remove all remaing recurring pricing on inactive plan
    # as those should NOT be used
    cr.execute("""
        DELETE FROM sale_subscription_pricing ssp
              USING sale_subscription_plan AS ssp_plan
              WHERE ssp.plan_id = ssp_plan.id AND ssp_plan.active = false
    """)
